@@ -5,7 +5,8 @@ from select import select
 import time
 
 iDevices = map(evdev.InputDevice, (evdev.list_devices()))
-iDevices = {dev.fd: dev for dev in iDevices if 1 in dev.capabilities()}
+iDevices = {dev.fd: dev for dev in iDevices if evdev.events.EV_KEY in dev.capabilities()}
+
 uDevices = {}
 for fd in iDevices:
     dev = iDevices[fd]
