@@ -28,6 +28,7 @@ from fenrirscreenreader.core import byteManager
 from fenrirscreenreader.core import attributeManager
 from fenrirscreenreader.core import barrierManager
 from fenrirscreenreader.core import remoteManager
+from fenrirscreenreader.core import sayAllManager
 from fenrirscreenreader.core import environment 
 from fenrirscreenreader.core.settingsData import settingsData
 from fenrirscreenreader.core import debug
@@ -50,7 +51,7 @@ class settingsManager():
                 break
             line = line.replace('\n','')
             if line.replace(" ","") == '':
-                continue            
+                continue
             if line.replace(" ","").startswith("#"):
                 continue
             if line.count("=") != 1:
@@ -103,7 +104,7 @@ class settingsManager():
     def getSetting(self, section, setting):
         value = ''
         try:
-            value = self.settingArgDict[section.lower()][setting.lower()]
+            value = self.settingArgDict[section][setting]
             return value
         except:
             pass
@@ -116,7 +117,7 @@ class settingsManager():
     def getSettingAsInt(self, section, setting):
         value = 0
         try:
-            value = int(self.settingArgDict[section.lower()][setting.lower()])
+            value = int(self.settingArgDict[section][setting])
             return value
         except Exception as e:
             pass
@@ -129,7 +130,7 @@ class settingsManager():
     def getSettingAsFloat(self, section, setting):
         value = 0.0
         try:
-            value = float(self.settingArgDict[section.lower()][setting.lower()])
+            value = float(self.settingArgDict[section][setting])
             return value
         except Exception as e:
             pass
@@ -142,10 +143,10 @@ class settingsManager():
     def getSettingAsBool(self, section, setting):
         value = False
         try:
-            value = self.settingArgDict[section.lower()][setting.lower()].upper() in ['1','YES','JA','TRUE']
+            value = self.settingArgDict[section][setting].upper() in ['1','YES','JA','TRUE']
             return value
         except Exception as e:
-            pass      
+            pass
         try:
             value = self.env['settings'].getboolean(section, setting)
         except:
@@ -154,8 +155,10 @@ class settingsManager():
 
     def loadDriver(self, driverName, driverType):
         try:
-            if self.env['runtime'][driverType] != None:
-                self.env['runtime'][driverType].shutdown(self.env)    
+            self.env['runtime'][driverType].shutdown(self.env)
+        except:
+            pass
+        try:
             driver_mod = module_utils.importModule(driverName,
               fenrirPath + "/" + driverType + '/' + driverName + '.py')
             self.env['runtime'][driverType] = driver_mod.driver()
@@ -170,10 +173,8 @@ class settingsManager():
                 self.env['runtime'][driverType].initialize(self.env)
             except Exception as e:
                 self.env['runtime']['debug'].writeDebugOut('(fallback) Loading Driver '  + driverType + ' (dummyDriver) FAILED:'+ str(e), debug.debugLevel.ERROR)
-            
+
     def shutdownDriver(self, driverType):
-        if self.env['runtime'][driverType] == None:
-            return
         try:
             self.env['runtime'][driverType].shutdown()
         except Exception as e:
@@ -194,14 +195,35 @@ class settingsManager():
                 self.env['input']['scriptKey'].append(key)
     def resetSettingArgDict(self):
         self.settingArgDict = {}
-    def setOptionArgDict(self, section, option, value):
-        section = section.lower()
-        option = option.lower()
+    def setOptionArgDict(self, section, setting, value):
+        #section = section.lower()
+        #setting = setting.lower()
         try:
             e = self.settingArgDict[section]
         except KeyError:
             self.settingArgDict[section] = {}
-        self.settingArgDict[section][option] = str(value)
+        try:
+            t = self.settings[section][setting]
+        except:
+            print(section,setting, 'not found')
+            return
+        try:
+            if isinstance(self.settings[section][setting], str):
+                v = str(value)
+            elif isinstance(self.settings[section][setting], bool):
+                if not value in ['True','False']:
+                    raise ValueError('could not convert string to bool: '+ value)
+            elif isinstance(self.settings[section][setting], int):
+                v = int(value)
+            elif isinstance(self.settings[section][setting], float):
+                v = float(value)
+            self.settingArgDict[section][setting] = str(value)
+        except Exception as e:
+            print('settingsManager:setOptionArgDict:Datatype missmatch: '+ section + '#' + setting + '=' +  value + ' Error:' +  str(e))
+            #self.env['runtime']['debug'].writeDebugOut('settingsManager:setOptionArgDict:Datatype missmatch: '+ section + '#' + setting + '=' +  value + ' Error:' +  str(e), debug.debugLevel.ERROR)
+            return
+
+
 
     def parseSettingArgs(self, settingArgs):
         for optionElem in settingArgs.split(';'):
@@ -210,8 +232,8 @@ class settingsManager():
             if len(optionElem.split('#',1)[1].split('=',1)) != 2:
                 continue
 
-            section = str(optionElem.split('#',1)[0]).lower()
-            option = str(optionElem.split('#',1)[1].split('=',1)[0]).lower()
+            section = str(optionElem.split('#',1)[0])
+            option = str(optionElem.split('#',1)[1].split('=',1)[0])
             value = optionElem.split('#',1)[1].split('=',1)[1]
             self.setOptionArgDict(section, option, value)
 
@@ -245,20 +267,20 @@ class settingsManager():
         if cliArgs.options != '':
             self.parseSettingArgs(cliArgs.options)
         if cliArgs.debug:
-            self.setOptionArgDict('general', 'debugLevel', 3)   
+            self.setSetting('general', 'debugLevel', 3)
         if cliArgs.print:
-            self.setOptionArgDict('general', 'debugLevel', 3)  
-            self.setOptionArgDict('general', 'debugMode', 'PRINT')
+            self.setSetting('general', 'debugLevel', 3)
+            self.setSetting('general', 'debugMode', 'PRINT')
         if cliArgs.emulated_pty:
-            self.setOptionArgDict('screen', 'driver', 'ptyDriver')  
-            self.setOptionArgDict('keyboard', 'driver', 'ptyDriver')  
+            self.setSetting('screen', 'driver', 'ptyDriver')
+            self.setSetting('keyboard', 'driver', 'ptyDriver')
             # TODO needs cleanup use dict
             #self.setOptionArgDict('keyboard', 'keyboardLayout', 'pty')
             self.setSetting('keyboard', 'keyboardLayout', 'pty')
-            self.setOptionArgDict('general', 'debugFile', '/tmp/fenrir-pty.log')
+            self.setSetting('general', 'debugFile', '/tmp/fenrir-pty.log')
         if cliArgs.emulated_evdev:
-            self.setOptionArgDict('screen', 'driver', 'ptyDriver')
-            self.setOptionArgDict('keyboard', 'driver', 'evdevDriver')
+            self.setSetting('screen', 'driver', 'ptyDriver')
+            self.setSetting('keyboard', 'driver', 'evdevDriver')
 
         self.setFenrirKeys(self.getSetting('general','fenrirKeys'))
         self.setScriptKeys(self.getSetting('general','scriptKeys'))
@@ -267,9 +289,9 @@ class settingsManager():
         environment['runtime']['debug'].initialize(environment)
 
         if not os.path.exists(self.getSetting('sound','theme') + '/soundicons.conf'):
-            if os.path.exists(soundRoot + self.getSetting('sound','theme')):  
+            if os.path.exists(soundRoot + self.getSetting('sound','theme')):
                 self.setSetting('sound', 'theme', soundRoot + self.getSetting('sound','theme'))
-                if os.path.exists(self.getSetting('sound','theme') + '/soundicons.conf'):  
+                if os.path.exists(self.getSetting('sound','theme') + '/soundicons.conf'):
                     environment['runtime']['settingsManager'].loadSoundIcons(self.getSetting('sound','theme'))
         else:
             environment['runtime']['settingsManager'].loadSoundIcons(self.getSetting('sound','theme'))
@@ -348,23 +370,23 @@ class settingsManager():
                     environment['runtime']['byteManager'].loadByteShortcuts(self.getSetting('keyboard','keyboardLayout'))
             else:
                 environment['runtime']['byteManager'].loadByteShortcuts(self.getSetting('keyboard','keyboardLayout'))
-                              
+
         environment['runtime']['cursorManager'] = cursorManager.cursorManager()
-        environment['runtime']['cursorManager'].initialize(environment)  
+        environment['runtime']['cursorManager'].initialize(environment)
         environment['runtime']['applicationManager'] = applicationManager.applicationManager()
-        environment['runtime']['applicationManager'].initialize(environment)  
+        environment['runtime']['applicationManager'].initialize(environment)
         environment['runtime']['textManager'] = textManager.textManager()
-        environment['runtime']['textManager'].initialize(environment)      
+        environment['runtime']['textManager'].initialize(environment)
         environment['runtime']['tableManager'] = tableManager.tableManager()
-        environment['runtime']['tableManager'].initialize(environment)    
+        environment['runtime']['tableManager'].initialize(environment)
         environment['runtime']['barrierManager'] = barrierManager.barrierManager()
-        environment['runtime']['barrierManager'].initialize(environment)              
-            
-        environment['runtime']['debug'].writeDebugOut('\/-------environment-------\/',debug.debugLevel.INFO, onAnyLevel=True)        
+        environment['runtime']['barrierManager'].initialize(environment)
+        environment['runtime']['sayAllManager'] = sayAllManager.sayAllManager()
+        environment['runtime']['sayAllManager'].initialize(environment)
+        environment['runtime']['debug'].writeDebugOut('\/-------environment-------\/',debug.debugLevel.INFO, onAnyLevel=True)
         environment['runtime']['debug'].writeDebugOut(str(environment), debug.debugLevel.INFO, onAnyLevel=True)
-        environment['runtime']['debug'].writeDebugOut('\/-------settings.conf-------\/', debug.debugLevel.INFO, onAnyLevel=True)        
+        environment['runtime']['debug'].writeDebugOut('\/-------settings.conf-------\/', debug.debugLevel.INFO, onAnyLevel=True)
         environment['runtime']['debug'].writeDebugOut(str(environment['settings']._sections) , debug.debugLevel.INFO, onAnyLevel=True)
-        environment['runtime']['debug'].writeDebugOut('\/-------self.settingArgDict-------\/',debug.debugLevel.INFO, onAnyLevel=True)        
+        environment['runtime']['debug'].writeDebugOut('\/-------self.settingArgDict-------\/',debug.debugLevel.INFO, onAnyLevel=True)
         environment['runtime']['debug'].writeDebugOut(str( self.settingArgDict) ,debug.debugLevel.INFO, onAnyLevel=True)
         return environment
-     
